"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var ReactDOM = require("react-dom");
var SteppableInput_1 = require("../SteppableInput/SteppableInput");
var Control_1 = require("../Control/Control");
var classnames = require("classnames");
var react_dom_1 = require("react-dom");
var withTheme_1 = require("../../utils/withTheme");
var ButtonIcon_1 = require("../ButtonIcon/ButtonIcon");
var is_before = require("date-fns/is_before");
var is_after = require("date-fns/is_after");
var pure_1 = require("../../utils/pure");
var Popover_1 = require("../Popover/Popover");
var with_defaults_1 = require("../../utils/with-defaults");
var DateFormatType;
(function (DateFormatType) {
    DateFormatType[DateFormatType["MDY"] = 0] = "MDY";
    DateFormatType[DateFormatType["DMY"] = 1] = "DMY";
})(DateFormatType = exports.DateFormatType || (exports.DateFormatType = {}));
var ActiveSection;
(function (ActiveSection) {
    ActiveSection[ActiveSection["Day"] = 0] = "Day";
    ActiveSection[ActiveSection["Month"] = 1] = "Month";
    ActiveSection[ActiveSection["Year"] = 2] = "Year";
})(ActiveSection || (ActiveSection = {}));
exports.DATE_INPUT = Symbol('DateInput');
var RawDateInput = /** @class */ (function (_super) {
    tslib_1.__extends(RawDateInput, _super);
    function RawDateInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isOpened: false,
        };
        _this.secondInput = false;
        _this.onIncrement = function () {
            _this.secondInput = false;
            var _a = _this.state, day = _a.day, month = _a.month, year = _a.year, activeSection = _a.activeSection;
            switch (activeSection) {
                case ActiveSection.Day: {
                    //day starts from 1 here and cannot be zero
                    var newDay = isDefined(day) ? (day + 1) % 32 || 1 : 1;
                    _this.updateStateTime(newDay, month, year);
                    break;
                }
                case ActiveSection.Month: {
                    //month starts from 1 here and cannot be zero
                    var newMonth = isDefined(month) ? (month + 1) % 13 || 1 : 1;
                    _this.updateStateTime(day, newMonth, year);
                    break;
                }
                case ActiveSection.Year: {
                    var newYear = isDefined(year) && year !== 9999 ? year + 1 : new Date().getFullYear();
                    _this.updateStateTime(day, month, newYear);
                    break;
                }
            }
        };
        _this.onDecrement = function () {
            _this.secondInput = false;
            var _a = _this.state, day = _a.day, month = _a.month, year = _a.year, activeSection = _a.activeSection;
            switch (activeSection) {
                case ActiveSection.Day: {
                    //day starts from 1 and cannot be zero
                    var newDay = isDefined(day) ? (day - 1) % 32 || 31 : 31;
                    _this.updateStateTime(newDay, month, year);
                    break;
                }
                case ActiveSection.Month: {
                    //month starts from 1 and cannot be zero
                    var newMonth = isDefined(month) ? (month - 1) % 13 || 12 : 12;
                    _this.updateStateTime(day, newMonth, year);
                    break;
                }
                case ActiveSection.Year: {
                    var newYear = isDefined(year) && year !== 0 ? year - 1 : new Date().getFullYear();
                    _this.updateStateTime(day, month, newYear);
                    break;
                }
            }
        };
        _this.onCalendarValueChange = function (date) {
            var _a = _this.props, onValueChange = _a.onValueChange, value = _a.value;
            _this.setState({
                isOpened: false,
            });
            if (onValueChange &&
                date &&
                date &&
                !isNaN(date.getTime()) &&
                (!value ||
                    isNaN(value.getTime()) ||
                    value.getFullYear() !== date.getFullYear() ||
                    value.getMonth() !== date.getMonth() ||
                    value.getDate() !== date.getDate())) {
                onValueChange(date);
            }
        };
        _this.onCalendarMouseDown = function (e) {
            //stop blur
            e.preventDefault();
        };
        _this.onClear = function () {
            _this.secondInput = false;
            _this.updateStateTime();
            var onClear = _this.props.onClear;
            onClear && onClear();
        };
        _this.onCalendarButtonMouseDown = function (e) {
            if (isDefined(_this.state.activeSection)) {
                e.preventDefault();
            }
        };
        _this.onDayMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: ActiveSection.Day,
                });
            }
        };
        _this.onMonthMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: ActiveSection.Month,
                });
            }
        };
        _this.onYearMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: ActiveSection.Year,
                });
            }
        };
        _this.onKeyDown = function (e) {
            var _a = _this.state, activeSection = _a.activeSection, day = _a.day, month = _a.month, year = _a.year;
            switch (e.keyCode) {
                case Control_1.KeyCode.Escape: {
                    if (_this.state.isOpened) {
                        _this.setState({
                            isOpened: false,
                        });
                    }
                    break;
                }
                case Control_1.KeyCode.Left: {
                    e.preventDefault(); //block h-scrolling
                    switch (activeSection) {
                        case ActiveSection.Month: //fallthrough
                        case ActiveSection.Year: {
                            _this.secondInput = false;
                            break;
                        }
                    }
                    _this.selectPreviousSection();
                    break;
                }
                case Control_1.KeyCode.Right: {
                    e.preventDefault(); //block h-scrolling
                    switch (activeSection) {
                        case ActiveSection.Day: //fallthrough
                        case ActiveSection.Month: {
                            _this.secondInput = false;
                            break;
                        }
                    }
                    _this.selectNextSection();
                    break;
                }
                case Control_1.KeyCode.Delete: //fallthrough
                case Control_1.KeyCode.Backspace: {
                    _this.secondInput = false;
                    switch (activeSection) {
                        case ActiveSection.Day: {
                            _this.updateStateTime(undefined, month, year);
                            break;
                        }
                        case ActiveSection.Month: {
                            _this.updateStateTime(day, undefined, year);
                            break;
                        }
                        case ActiveSection.Year: {
                            _this.updateStateTime(day, month, undefined);
                            break;
                        }
                    }
                    break;
                }
                default: {
                    var number = Control_1.KEY_CODE_NUM_MAP[e.keyCode];
                    if (isDefined(number)) {
                        _this.handleDigitKeyDown(number);
                    }
                }
            }
        };
        _this.onSteppableInputClick = function (e) {
            if (_this.state.isOpened) {
                var calendarButtonDOMNode = ReactDOM.findDOMNode(_this.calendarButtonRef);
                var thisNode = ReactDOM.findDOMNode(_this);
                if (calendarButtonDOMNode === null ||
                    calendarButtonDOMNode instanceof Text ||
                    thisNode === null ||
                    thisNode instanceof Text) {
                    return;
                }
                if (SteppableInput_1.checkParentsUpTo(e.target, calendarButtonDOMNode, thisNode)) {
                    //clicked on calendar button
                    _this.setState({
                        isOpened: false,
                    });
                }
                else {
                    _this.setState({
                        isOpened: true,
                    });
                }
            }
            else {
                _this.setState({
                    isOpened: true,
                });
            }
        };
        _this.onBlur = function (e) {
            _this.secondInput = false;
            _this.setState({
                activeSection: undefined,
                isOpened: false,
            });
            _this.props.onBlur && _this.props.onBlur();
        };
        _this.onFocus = function (e) {
            _this.secondInput = false;
            if (!isDefined(_this.state.activeSection)) {
                _this.setState({
                    activeSection: _this.getDefaultActiveSection(_this.props.dateFormatType),
                });
            }
        };
        return _this;
    }
    RawDateInput.prototype.componentWillMount = function () {
        var value = this.props.value;
        if (value) {
            this.setState(getValuesFromDate(value));
        }
    };
    RawDateInput.prototype.componentWillReceiveProps = function (newProps) {
        if (this.props.value !== newProps.value && isDefined(newProps.value)) {
            var month = void 0;
            var day = void 0;
            var year = void 0;
            if (newProps.value !== null && !isNaN(newProps.value.getTime())) {
                var result = getValuesFromDate(newProps.value);
                month = result.month;
                day = result.day;
                year = result.year;
            }
            this.setState({
                month: month,
                day: day,
                year: year,
            });
        }
    };
    RawDateInput.prototype.render = function () {
        var _this = this;
        var _a, _b;
        var _c = this.props, isDisabled = _c.isDisabled, error = _c.error, clearIcon = _c.clearIcon, calendarIcon = _c.calendarIcon, Calendar = _c.Calendar, incrementIcon = _c.incrementIcon, value = _c.value, decrementIcon = _c.decrementIcon, theme = _c.theme, ButtonIcon = _c.ButtonIcon, SteppableInput = _c.SteppableInput, dateFormatType = _c.dateFormatType;
        var _d = this.state, month = _d.month, day = _d.day, year = _d.year, activeSection = _d.activeSection;
        var yearClassName = classnames(theme.section, (_a = {},
            _a[theme.section_isActive] = !isDisabled && activeSection === ActiveSection.Year,
            _a));
        var onClear;
        // check if "X" clear button should be visible - at least one part of date should be set
        if ((isDefined(value) && value !== null) || isDefined(day) || isDefined(month) || isDefined(year)) {
            onClear = this.onClear;
        }
        var innerClassName = classnames(theme.inner, (_b = {},
            _b[theme.inner_isFilled] = value && !isNaN(value.getTime()),
            _b));
        return (React.createElement(SteppableInput, { isDisabled: isDisabled, theme: theme.SteppableInput, onClear: onClear, error: error, decrementIcon: decrementIcon, incrementIcon: incrementIcon, clearIcon: clearIcon, onIncrement: this.onIncrement, onDecrement: this.onDecrement, onBlur: this.onBlur, onFocus: this.onFocus, onKeyDown: this.onKeyDown, onClick: this.onSteppableInputClick },
            React.createElement("div", { className: innerClassName },
                dateFormatType === DateFormatType.DMY && this.renderDay(),
                dateFormatType === DateFormatType.MDY && this.renderMonth(),
                React.createElement("span", { className: theme.separator }, "/"),
                dateFormatType === DateFormatType.DMY && this.renderMonth(),
                dateFormatType === DateFormatType.MDY && this.renderDay(),
                React.createElement("span", { className: theme.separator }, "/"),
                React.createElement("span", { className: yearClassName, onMouseDown: this.onYearMouseDown }, this.format(year, ActiveSection.Year))),
            Calendar &&
                calendarIcon && (React.createElement(ButtonIcon, { isFlat: true, ref: function (el) { return (_this.calendarButtonRef = el); }, isDisabled: isDisabled, tabIndex: -1, icon: calendarIcon, onMouseDown: this.onCalendarButtonMouseDown, theme: theme.CalendarButtonIcon })),
            this.renderCalendar()));
    };
    RawDateInput.prototype.renderDay = function () {
        var _a;
        var _b = this.props, isDisabled = _b.isDisabled, theme = _b.theme;
        var _c = this.state, activeSection = _c.activeSection, day = _c.day;
        var dayClassName = classnames(theme.section, (_a = {},
            _a[theme.section_isActive] = !isDisabled && activeSection === ActiveSection.Day,
            _a));
        return (React.createElement("span", { className: dayClassName, onMouseDown: this.onDayMouseDown }, this.format(day, ActiveSection.Day)));
    };
    RawDateInput.prototype.renderMonth = function () {
        var _a;
        var _b = this.props, isDisabled = _b.isDisabled, theme = _b.theme;
        var _c = this.state, activeSection = _c.activeSection, month = _c.month;
        var monthClassName = classnames(theme.section, (_a = {},
            _a[theme.section_isActive] = !isDisabled && activeSection === ActiveSection.Month,
            _a));
        return (React.createElement("span", { className: monthClassName, onMouseDown: this.onMonthMouseDown }, this.format(month, ActiveSection.Month)));
    };
    RawDateInput.prototype.renderCalendar = function () {
        var _a = this.props, target = _a.target, Calendar = _a.Calendar, Popover = _a.Popover, value = _a.value, min = _a.min, max = _a.max, theme = _a.theme;
        var isOpened = this.state.isOpened;
        if (!Calendar) {
            return null;
        }
        var calendar = (React.createElement(Calendar, { value: value, min: min, max: max, onMouseDown: this.onCalendarMouseDown, onValueChange: this.onCalendarValueChange }));
        if (target) {
            return react_dom_1.createPortal(isOpened ? calendar : null, target);
        }
        else {
            return (React.createElement(Popover, { theme: theme.Popover, anchor: this, closeOnClickAway: true, onMouseDown: this.onCalendarMouseDown, isOpened: isOpened }, calendar));
        }
    };
    RawDateInput.prototype.format = function (value, section) {
        if (isDefined(value)) {
            switch (section) {
                //maybe we should use left-pad here? ;)
                case ActiveSection.Day: //fallthrough
                case ActiveSection.Month: {
                    //day and month are 2 digits
                    return "" + (value >= 0 && value < 10 ? 0 : '') + value;
                }
                case ActiveSection.Year: {
                    if (value < 10) {
                        return "000" + value;
                    }
                    else if (value < 100) {
                        return "00" + value;
                    }
                    else if (value < 1000) {
                        return "0" + value;
                    }
                    else {
                        return "" + value;
                    }
                }
            }
        }
        switch (section) {
            case ActiveSection.Day: {
                return 'dd';
            }
            case ActiveSection.Month: {
                return 'mm';
            }
            case ActiveSection.Year: {
                return 'yyyy';
            }
        }
    };
    RawDateInput.prototype.updateStateTime = function (day, month, year) {
        var _a = this.props, onValueChange = _a.onValueChange, value = _a.value, min = _a.min, max = _a.max;
        var canBuildValue = isDefined(day) && isDefined(month) && isDefined(year);
        var newValueDiffers = canBuildValue &&
            isDefined(month) &&
            (!isDefined(value) ||
                value === null ||
                value.getDate() !== day ||
                value.getMonth() !== month - 1 ||
                value.getFullYear() !== year);
        if (canBuildValue) {
            if (newValueDiffers && onValueChange && isDefined(year) && isDefined(month) && isDefined(day)) {
                var date = new Date(year, month - 1, day);
                //check new date
                var wasAdjusted = !(date.getFullYear() === year &&
                    date.getMonth() === month - 1 &&
                    date.getDate() === day);
                var isOutOfBounds = (min && is_before(date, min)) || (max && is_after(date, max));
                if (!wasAdjusted && !isOutOfBounds) {
                    //everything is ok and value hasn't been adjusted
                    onValueChange(date);
                }
                else {
                    //too "smart" Date constructor has adjusted our value - date is actually invalid
                    //or date is out of bounds
                    onValueChange(undefined);
                    this.setState({
                        day: day,
                        month: month,
                        year: year,
                    });
                }
            }
        }
        else {
            if (isDefined(this.props.value)) {
                onValueChange && onValueChange(undefined);
            }
            this.setState({
                day: day,
                month: month,
                year: year,
            });
        }
    };
    RawDateInput.prototype.getDefaultActiveSection = function (dateFormatType) {
        switch (dateFormatType) {
            case DateFormatType.DMY: {
                return ActiveSection.Day;
            }
            case DateFormatType.MDY: {
                return ActiveSection.Month;
            }
        }
    };
    RawDateInput.prototype.selectNextSection = function () {
        var activeSection = this.state.activeSection;
        var dateFormatType = this.props.dateFormatType;
        switch (activeSection) {
            case ActiveSection.Day: {
                switch (dateFormatType) {
                    case DateFormatType.DMY: {
                        this.setState({
                            activeSection: ActiveSection.Month,
                        });
                        break;
                    }
                    case DateFormatType.MDY: {
                        this.setState({
                            activeSection: ActiveSection.Year,
                        });
                        break;
                    }
                }
                break;
            }
            case ActiveSection.Month: {
                switch (dateFormatType) {
                    case DateFormatType.DMY: {
                        this.setState({
                            activeSection: ActiveSection.Year,
                        });
                        break;
                    }
                    case DateFormatType.MDY: {
                        this.setState({
                            activeSection: ActiveSection.Day,
                        });
                        break;
                    }
                }
                break;
            }
        }
    };
    RawDateInput.prototype.selectPreviousSection = function () {
        var activeSection = this.state.activeSection;
        var dateFormatType = this.props.dateFormatType;
        switch (activeSection) {
            case ActiveSection.Day: {
                switch (dateFormatType) {
                    case DateFormatType.MDY: {
                        this.setState({
                            activeSection: ActiveSection.Month,
                        });
                        break;
                    }
                }
                break;
            }
            case ActiveSection.Month: {
                switch (dateFormatType) {
                    case DateFormatType.DMY: {
                        this.setState({
                            activeSection: ActiveSection.Day,
                        });
                        break;
                    }
                }
                break;
            }
            case ActiveSection.Year: {
                switch (dateFormatType) {
                    case DateFormatType.DMY: {
                        this.setState({
                            activeSection: ActiveSection.Month,
                        });
                        break;
                    }
                    case DateFormatType.MDY: {
                        this.setState({
                            activeSection: ActiveSection.Day,
                        });
                        break;
                    }
                }
                break;
            }
        }
    };
    RawDateInput.prototype.handleDigitKeyDown = function (digit) {
        var _a = this.state, day = _a.day, month = _a.month, year = _a.year;
        switch (this.state.activeSection) {
            case ActiveSection.Day: {
                if (this.secondInput) {
                    var newDay = void 0;
                    if (isDefined(day) && day < 3) {
                        newDay = Number("" + day + digit);
                    }
                    else if (day === 3) {
                        newDay = Math.min(Number("" + day + digit), 31);
                    }
                    else {
                        newDay = digit;
                    }
                    this.updateStateTime(newDay, month, year);
                    this.selectNextSection();
                    this.secondInput = false;
                }
                else {
                    this.updateStateTime(digit, month, year);
                    if (digit > 3) {
                        this.selectNextSection();
                        this.secondInput = false;
                    }
                    else {
                        this.secondInput = true;
                    }
                }
                break;
            }
            case ActiveSection.Month: {
                if (this.secondInput) {
                    var newMonth = void 0;
                    if (isDefined(month) && month < 1) {
                        newMonth = Number("" + month + digit);
                    }
                    else if (month === 1) {
                        newMonth = Math.min(Number("" + month + digit), 12);
                    }
                    else {
                        newMonth = digit;
                    }
                    this.updateStateTime(day, newMonth, year);
                    this.selectNextSection();
                    this.secondInput = false;
                }
                else {
                    this.updateStateTime(day, digit, year);
                    if (digit > 1) {
                        this.selectNextSection();
                        this.secondInput = false;
                    }
                    else {
                        this.secondInput = true;
                    }
                }
                break;
            }
            case ActiveSection.Year: {
                if (this.secondInput) {
                    var newYear = "" + year + digit;
                    if (isDefined(year) && year >= 1000) {
                        newYear = newYear.substr(1);
                    }
                    this.updateStateTime(day, month, Number(newYear));
                }
                else {
                    this.updateStateTime(day, month, digit);
                    this.secondInput = true;
                }
                break;
            }
        }
    };
    RawDateInput = tslib_1.__decorate([
        pure_1.PURE
    ], RawDateInput);
    return RawDateInput;
}(React.Component));
exports.DateInput = withTheme_1.withTheme(exports.DATE_INPUT)(with_defaults_1.withDefaults({
    SteppableInput: SteppableInput_1.SteppableInput,
    ButtonIcon: ButtonIcon_1.ButtonIcon,
    Popover: Popover_1.Popover,
    dateFormatType: DateFormatType.DMY,
})(RawDateInput));
function getValuesFromDate(date) {
    return {
        month: date.getMonth() + 1,
        day: date.getDate(),
        year: date.getFullYear(),
    };
}
function isDefined(value) {
    return typeof value !== 'undefined';
}
//# sourceMappingURL=DateInput.js.map